#!/bin/sh

ZPKG_GDRIVE_BIN="/usr/local/zy-pkgs/bin/grive"
ZPKG_GDRIVE_UPDATE_SCRIPT="/usr/local/zy-pkgs/bin/gdrive_update.sh"
ZPKG_GDRIVE_CONFIG_DB="/usr/local/zy-pkgs/config/GoogleDrive/GoogleDrive.db"
ZPKG_GDRIVE_TOKEN=".grive"
ZPKG_GDRIVE_STATE=".grive_state"
ZPKG_GDRIVE_STATE_DRY=".grive_state_dry"
ZPKG_GDRIVE_SYNC=".grive_syncing"
ZPKG_GDRIVE_USER=".grive_user"
ZPKG_GDRIVE_UPDATE=".grive_update"
ZPKG_GDRIVE_LAST_SYNC=".grive_last_sync"
ZPKG_GDRIVE_MAX_RETRY=3

read_json_config()
{
	JSON=`cat $1 | sed -e 's/[{}]/''/g' | awk -v k="text" '{n=split($0,a,","); for (i=1; i<=n; i++) print a[i]}' | grep "\"$2\":" | sed 's/:/ /1' | awk -F" " '{ print $2 }'`
}

write_config()
{
	# If the request comes from auth, read config from .grive_state_dry; otherwise, read from .grive_state
	if [ "$1" = "auth" ]; then
		ZPKG_GDRIVE_STATE_FILE=${ZPKG_GDRIVE_STATE_DRY}
	else
		ZPKG_GDRIVE_STATE_FILE=${ZPKG_GDRIVE_STATE}
	fi

	read_json_config "${ZPKG_GDRIVE_STATE_FILE}" "user_account"
	_user_account=`echo $JSON | tr -d \"`
	read_json_config "${ZPKG_GDRIVE_STATE_FILE}" "quota_total"
	_quota_total=$JSON
	read_json_config "${ZPKG_GDRIVE_STATE_FILE}" "quota_used"
	_quota_used=$JSON
	read_json_config "${ZPKG_GDRIVE_STATE_FILE}" "quota_trash"
	_quota_trash=$JSON
	read_json_config "${ZPKG_GDRIVE_STATE_FILE}" "change_stamp"
	_change_stamp=$JSON

	# If the request comes from auth, write config; otherwise, update config
	if [ "$1" = "auth" ]; then
		# Check account is duplicated or not
		ACCOUNT_EXIST=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM Account_Info WHERE account='${_user_account}'"`
		if [ $ACCOUNT_EXIST -ne 0 ]; then
			echo "Account Exists"
			rm -f "${ZPKG_GDRIVE_TOKEN}"
			rm -f "${ZPKG_GDRIVE_STATE_DRY}"
			exit 1
		fi
		# Write config to GoogleDrive.db
		sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} \
		"INSERT INTO Account_Info (account, status, sync_path, user, quota_total, quota_used, quota_trash, change_stamp) VALUES \
		('${_user_account}', '$2', '${ZPKG_GDRIVE_SYNC_FOLDER}', '$3', '${_quota_total}', '${_quota_used}', '${_quota_trash}', '${_change_stamp}')"

		# Write binding user to .gdrive_user
		USER_ID=`id -u "$3"`
		MOUNT_POINT=`df -P "${ZPKG_GDRIVE_SYNC_FOLDER}" | tail -1 | cut -d' ' -f 1`
		echo "$3" > ${ZPKG_GDRIVE_USER}
		echo "${USER_ID}" >> ${ZPKG_GDRIVE_USER}
		echo "${MOUNT_POINT}" >> ${ZPKG_GDRIVE_USER}
	else
		# Update config to GoogleDrive.db
		sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} \
		"UPDATE Account_Info SET \
		quota_total='${_quota_total}', \
		quota_used='${_quota_used}', \
		quota_trash='${_quota_trash}', \
		change_stamp='${_change_stamp}' \
		WHERE account='${_user_account}'"
	fi
}

case "$1" in
	auth)
		ZPKG_GDRIVE_SYNC_FOLDER=`echo $2 | tr -d \"`
		ZPKG_GDRIVE_NAS_USER=`echo $3 | tr -d \"`
		ZPKG_GDRIVE_AUTH_CODE=$4

		if [ ! -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_TOKEN}" ]; then
			[ -f "${ZPKG_GDRIVE_SYNC_FOLDER}" ] || mkdir -p "${ZPKG_GDRIVE_SYNC_FOLDER}"
			cd "${ZPKG_GDRIVE_SYNC_FOLDER}"
			[ -f "${ZPKG_GDRIVE_SYNC}" ] || touch "${ZPKG_GDRIVE_SYNC}"
			${ZPKG_GDRIVE_BIN} --dry-run -a ${ZPKG_GDRIVE_AUTH_CODE} > /dev/null 2>&1
			if [ $? != 0 ]; then
				[ ! -f "${ZPKG_GDRIVE_SYNC}" ] || rm -f "${ZPKG_GDRIVE_SYNC}"
				exit 1
			fi
			[ ! -f "${ZPKG_GDRIVE_SYNC}" ] || rm -f "${ZPKG_GDRIVE_SYNC}"
			write_config "auth" "0" "${ZPKG_GDRIVE_NAS_USER}"
			echo "Authorization Success"
			chmod -R 777 .
			chown-nobody -R "${ZPKG_GDRIVE_NAS_USER}.everyone" .
		else
			echo "The token file already exists"
			exit 1
		fi
	;;
	sync)
		ZPKG_GDRIVE_SYNC_FOLDER=$2

		if [ -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_TOKEN}" ]; then
			cd "${ZPKG_GDRIVE_SYNC_FOLDER}"
			[ -f "${ZPKG_GDRIVE_SYNC}" ] || touch "${ZPKG_GDRIVE_SYNC}"
			if [ -f "${ZPKG_GDRIVE_UPDATE}" ]; then 
				TIMESTAMP_BEFORE=`cat ${ZPKG_GDRIVE_UPDATE}`
			else
				TIMESTAMP_BEFORE=0
			fi

			# Count for maximum try, if exceed maximum try, give up syncing
			i=0
			false
			while [ $? -ne 0 -a $i -lt $ZPKG_GDRIVE_MAX_RETRY ]; do
				i=$(($i+1))
				${ZPKG_GDRIVE_BIN} > /dev/null 2>&1

				# Return code 113 means that the user exceeds quota
				if [ $? -eq 113 ]; then
					GDRIVE_ACCOUNT=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT account FROM Account_Info WHERE sync_path='${ZPKG_GDRIVE_SYNC_FOLDER}'"`
					NAS_USER=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT user FROM Account_Info WHERE sync_path='${ZPKG_GDRIVE_SYNC_FOLDER}'"`
					zylogger -s 17 -p 4 -f 17 "GoogleDrive: Account \"${GDRIVE_ACCOUNT}\" stop syncing due to exceeding user \"${NAS_USER}\" quota"
					cat ${ZPKG_GDRIVE_SYNC} | xargs rm -f
					[ ! -f "${ZPKG_GDRIVE_SYNC}" ] || rm -f "${ZPKG_GDRIVE_SYNC}"
					exit 1
				fi
			done

	 		if [ $i -eq $ZPKG_GDRIVE_MAX_RETRY ]; then
				echo "Hit maximum number of retries, giving up syncing."
				cat ${ZPKG_GDRIVE_SYNC} | xargs rm -f
				[ ! -f "${ZPKG_GDRIVE_SYNC}" ] || rm -f "${ZPKG_GDRIVE_SYNC}"
				exit 1
			fi

			if [ -f "${ZPKG_GDRIVE_UPDATE}" ]; then
				TIMESTAMP_AFTER=`cat ${ZPKG_GDRIVE_UPDATE}`
			else
				TIMESTAMP_AFTER=0
			fi

			# If the value of timestamp doesn't change, remove .gdrive_update
			if [ $TIMESTAMP_BEFORE -ne 0 ] && [ $TIMESTAMP_AFTER -ne 0 ] && [ $TIMESTAMP_BEFORE -eq $TIMESTAMP_AFTER ]; then
				rm -f "${ZPKG_GDRIVE_UPDATE}"
			fi

			[ ! -f "${ZPKG_GDRIVE_SYNC}" ] || rm -f "${ZPKG_GDRIVE_SYNC}"
			write_config "sync"
		else
			echo "The token file doesn't exist"
			exit 1
		fi
	;;
	dry-run)
		ZPKG_GDRIVE_SYNC_FOLDER=$2

		if [ -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_TOKEN}" ]; then
			cd "${ZPKG_GDRIVE_SYNC_FOLDER}"
			i=0
			false
			while [ $? -ne 0 -a $i -lt $ZPKG_GDRIVE_MAX_RETRY ]; do
				i=$(($i+1))
				${ZPKG_GDRIVE_BIN} --dry-run > /dev/null 2>&1 
			done

	 		if [ $i -eq $ZPKG_GDRIVE_MAX_RETRY ]; then
				echo "-1"
				exit 1
			fi

			read_json_config "${ZPKG_GDRIVE_STATE_DRY}" "change_stamp"
			echo "$JSON"

		else
			echo "-1"
			exit 1
		fi
	;;
	set)
		ZPKG_GDRIVE_ACCOUNT=$2
		ZPKG_GDRIVE_ACCOUNT_STATUS=$3

		# Set account status
		sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "UPDATE Account_Info SET status='${ZPKG_GDRIVE_ACCOUNT_STATUS}' WHERE account='${ZPKG_GDRIVE_ACCOUNT}'"
	;;
	delete)
		ZPKG_GDRIVE_ACCOUNT=$2
		ZPKG_GDRIVE_DELETE_FOLDER=$3

		# Get sync path
		ZPKG_GDRIVE_SYNC_FOLDER=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT sync_path FROM Account_Info WHERE account='${ZPKG_GDRIVE_ACCOUNT}'"`

		if [ ${ZPKG_GDRIVE_DELETE_FOLDER} = "yes" ]; then
			rm -rf "${ZPKG_GDRIVE_SYNC_FOLDER}"
		else
			rm -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_TOKEN}"
			rm -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_USER}"
			rm -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_STATE}"
			rm -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_STATE_DRY}"
			rm -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_SYNC}"
			rm -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_UPDATE}"
			rm -f "${ZPKG_GDRIVE_SYNC_FOLDER}/${ZPKG_GDRIVE_LAST_SYNC}"
		fi
		
		# Remove account info from GoogleDrive.db
		sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "DELETE FROM Account_Info WHERE account='${ZPKG_GDRIVE_ACCOUNT}'"
	;;
	update)
		ZPKG_GDRIVE_UPDATE_PERIOD=$2

		# Set update period
		sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "UPDATE Update_Period SET period='${ZPKG_GDRIVE_UPDATE_PERIOD}'"

		killall -9 gdrive_update.sh
		start-stop-daemon -b -S -N 5 -x ${ZPKG_GDRIVE_UPDATE_SCRIPT}
	;;
	*)
		echo "Usage: $0 {auth|sync|dry-run|set|delete|update}"
		exit 1
esac

exit 0
