#!/bin/sh

ZPKG_GDRIVE_CONTROL_SCRIPT="/usr/local/zy-pkgs/bin/gdrive_control.sh"
ZPKG_GDRIVE_CONFIG_DB="/usr/local/zy-pkgs/config/GoogleDrive/GoogleDrive.db"
ZPKG_GDRIVE_TOKEN=".grive"
ZPKG_GDRIVE_STATE=".grive_state"
ZPKG_GDRIVE_SYNC=".grive_syncing"
ZPKG_GDRIVE_USER=".grive_user"
ZPKG_GDRIVE_UPDATE=".grive_update"
ZPKG_GDRIVE_MAX_RUNNING=3
ZPKG_GDRIVE_UPDATE_PERIOD=15

read_json_config()
{
	JSON=`cat $1 | sed -e 's/[{}]/''/g' | awk -v k="text" '{n=split($0,a,","); for (i=1; i<=n; i++) print a[i]}' | grep "\"$2\":" | sed 's/:/ /1' | awk -F" " '{ print $2 }'`
}

# Init GoogleDrive.db
sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} \
"CREATE TABLE IF NOT EXISTS Account_Info \
(account TEXT PRIMARY KEY NOT NULL, status INTEGER NOT NULL, sync_path TEXT UNIQUE NOT NULL, user TEXT NOT NULL, \
quota_total INTEGER NOT NULL, quota_used INTEGER NOT NULL, quota_trash INTEGER NOT NULL, change_stamp INTEGER NOT NULL)"

# Check Updatoe_Period table exists or not, if not, create it and set default value; otherwise apply value to UPDATE_INTERVAL
UPDATE_TABLE_EXIST=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM sqlite_master WHERE type='table' and name='Update_Period'"`
if [ $UPDATE_TABLE_EXIST -eq 0 ]; then
	sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "CREATE TABLE IF NOT EXISTS Update_Period (period INTEGER NOT NULL)"
	sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "INSERT INTO Update_Period VALUES ('$ZPKG_GDRIVE_UPDATE_PERIOD')"
else
	ZPKG_GDRIVE_UPDATE_PERIOD=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT period FROM Update_Period LIMIT 1"`
fi

UPDATE_INTERVAL=$(($ZPKG_GDRIVE_UPDATE_PERIOD*60)) 

while [ 1 ]; do

	sleep $UPDATE_INTERVAL

	ACCOUNT_TABLE_EXIST=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM sqlite_master WHERE type='table' and name='Account_Info'"`
	if [ $ACCOUNT_TABLE_EXIST -ne 0 ]; then

		ENABLED_COUNT=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT COUNT(*) FROM Account_Info WHERE status='1'"`
		if [ $? -ne 0 ] || [ $ENABLED_COUNT -eq 0 ]; then
			 continue
		else
			for index in `seq 1 $ENABLED_COUNT`; do

				# Check current running grive, if exceed maximum, check every 3 minutes to see the resource have been released or not
				RUNNING_GRIVE=`ps | grep -c [g]rive`
				while [ $RUNNING_GRIVE -ge $ZPKG_GDRIVE_MAX_RUNNING ]; do
					sleep 180
					RUNNING_GRIVE=`ps | grep -c [g]rive`
				done

				SYNC_PATH=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} \
						  "SELECT sync_path FROM Account_Info WHERE status='1' ORDER BY rowid ASC LIMIT 1 OFFSET $(($index-1))"`
				
				if [ -z "${SYNC_PATH}" ] || [ -f "${SYNC_PATH}/${ZPKG_GDRIVE_SYNC}" ] || [ ! -f "${SYNC_PATH}/${ZPKG_GDRIVE_TOKEN}" ]; then
					continue
				else
					USER=`awk 'NR==1' "${SYNC_PATH}/${ZPKG_GDRIVE_USER}"`
					MOUNT_POINT=`awk 'NR==3' "${SYNC_PATH}/${ZPKG_GDRIVE_USER}"`

					# Check user
					USER_CHECK=`id "${USER}"`
					if [ $? -ne 0 ]; then
						GDRIVE_ACCOUNT=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT account FROM Account_Info WHERE sync_path='${SYNC_PATH}'"`
						NAS_USER=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT user FROM Account_Info WHERE sync_path='${SYNC_PATH}'"`
						${ZPKG_GDRIVE_CONTROL_SCRIPT} delete "${GDRIVE_ACCOUNT}" "no" &
						zylogger -s 17 -p 4 -f 17 "GoogleDrive: Account \"${GDRIVE_ACCOUNT}\" stop syncing due to user \"${NAS_USER}\" is deleted"
						continue
					fi

					# Check quota
					QUOTA_CHECK=`repquota -n "${MOUNT_POINT}" | grep "${USER_UID}" | grep +`
					if [ -n "${QUOTA_CHECK}" ]; then
						GDRIVE_ACCOUNT=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT account FROM Account_Info WHERE sync_path='${SYNC_PATH}'"`
						NAS_USER=`sqlite3 ${ZPKG_GDRIVE_CONFIG_DB} "SELECT user FROM Account_Info WHERE sync_path='${SYNC_PATH}'"`
						zylogger -s 17 -p 4 -f 17 "GoogleDrive: Account \"${GDRIVE_ACCOUNT}\" stop syncing due to exceeding user \"${NAS_USER}\" quota"
						continue
					fi

					if [ ! -f "${SYNC_PATH}/${ZPKG_GDRIVE_STATE}" ] || [ -f "${SYNC_PATH}/${ZPKG_GDRIVE_UPDATE}" ]; then
						${ZPKG_GDRIVE_CONTROL_SCRIPT} sync "${SYNC_PATH}" &
					else
						_remote_change_stamp=`${ZPKG_GDRIVE_CONTROL_SCRIPT} dry-run "${SYNC_PATH}"`
						read_json_config "${SYNC_PATH}/${ZPKG_GDRIVE_STATE}" "change_stamp"
						_local_change_stamp=$JSON
						if [ $_remote_change_stamp -gt $_local_change_stamp ]; then
							${ZPKG_GDRIVE_CONTROL_SCRIPT} sync "${SYNC_PATH}" &
						else
							continue
						fi
					fi
				fi
			done
		fi
	else
		continue
	fi
	
done

exit 0
